Attribute VB_Name = "MainCode"
' SQL to Access Export program
' Utility for creating an Access database from a SQL table

' Suggested enhancements:
'   1 - Add code to ZIP the database after it has been created.
'   2 - Currently it uses DAO, convert it to ADO.

Option Explicit

'API Declarations for dealing with INI files:
Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Declare Function WritePrivateProfileString Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpString As Any, ByVal lpFileName As String) As Long

'Important variables:
Dim dbLocal As Database     'Destination database (Access)
Dim dbSQL As Database       'Source database (MS SQL Server)
Dim sDBLocalPath As String  'Path to destination database
Dim sINIPath As String      'Path to INI File

Sub Main()
    Dim sConnect As String      'SQL Server Connect String
    Dim nTables As Integer      'Number of tables to transfer
    Dim nCurTable As Integer    'Counter for current table
    Dim sSQLTable As String     'Name of the SQL table
    Dim sMDBTable As String     'Name of the Access table
    Dim sSection As String      'INI file [Section]
    Dim nCommand As Integer     'Commands can be run on the
    Dim sCommand As String      'Access database (i.e. create index)
        
    'GET INI FILE NAME
    sINIPath = App.Path & "\SQLXPORT.ini"
    If Trim$(Command$) <> "" Then sINIPath = Command$
    
    'READ INFO FROM THE INI FILE
    sDBLocalPath = sGetINIString(sINIPath, "General", "DBFile", "test.mdb")
    sConnect = sGetINIString(sINIPath, "General", "SQLConnect", "ODBC;")
    nTables = CInt(sGetINIString(sINIPath, "Tables", "Tables", "0"))
    
    'SHOW FORM AND CONNECT TO SQL SERVER
    frmWait.Show
    frmWait.lblWait = "Connecting to SQL Server..."
    DoEvents
    On Error GoTo MainError
    Set dbSQL = OpenDatabase("", False, True, sConnect)
        
    'CALL FUNCTION TO CREATE MDB FILE
    frmWait.lblWait = "Creating MDB file..."
    DoEvents
    CreateLocalFile
    DoEvents
    
    'CREATE TABLES IN THE NEW MDB FILE
    For nCurTable = 1 To nTables
        sSQLTable = sGetINIString(sINIPath, "tables", "table" & nCurTable & "SQL", "none")
        sMDBTable = sGetINIString(sINIPath, "tables", "table" & nCurTable & "MDB", "none")
        sSection = sGetINIString(sINIPath, "tables", "table" & nCurTable & "INI", "none")
        frmWait.lblWait = "Transferring " & sMDBTable
        TransferTable sSQLTable, sMDBTable, sSection
    Next nCurTable
                
    'THE DATABASE HAS BEEN CREATED, RUN COMMANDS ON IT IF NECESSARY
    nCommand = 0
    sCommand = ""
    While sCommand <> "?"
        If nCommand <> 0 Then dbLocal.Execute sCommand
        nCommand = nCommand + 1
        sCommand = sGetINIString(sINIPath, "General", "Command" & nCommand, "?")
    Wend
       
    'CLOSE EVERYTHING DOWN AND END
    frmWait.lblWait = "Disconnecting..."
    dbLocal.Close
    dbSQL.Close
    Unload frmWait
    DoEvents
    End

MainError:
    frmWait.Hide
    Screen.MousePointer = vbDefault
    WriteErrMsg "Main - Error " & Err & ": " & Error
    End
    Exit Sub

End Sub
Sub TransferTable(sSQLTable As String, sLocalTable As String, sSection As String)

'This function transfers data from SQL server to an Access table
    
    Dim rstemp As Recordset 'SQL recordset
    Dim aTable As Recordset 'Access table
    Dim nTemp As Integer    '  Counter
    Dim nFields As Integer  '  variables
    Dim nCount As Integer   '
    Dim sSQL As String      'SQL statement
    Dim sTemp As String
    

On Error GoTo TRTError:
    
    frmWait.ProgressBar1.Visible = True
    nCount = 0
    frmWait.ProgressBar1.Min = 0
    
    'The user can either transfer a SQL table as-is,
    'or the results of a query involving multiple tables.
    'This next IF statement determines which and sets up the
    'SQL statement- either "Select * from table" or the
    'user-defined SQL statement.
    
    sSQL = sGetINIString(sINIPath, sSection, "SQL", "")
    If sSQL = "" Then
        Set rstemp = dbSQL.OpenRecordset("Select Count(*) from " & sSQLTable, dbOpenSnapshot, dbForwardOnly)
        frmWait.ProgressBar1.Max = CInt(Trim$(" 0" & rstemp.Fields(0)))
        rstemp.Close
        DoEvents
    Else
        nTemp = 2
        sTemp = sGetINIString(sINIPath, sSection, "SQL" & nTemp, "")
        While sTemp <> ""
            If sTemp <> "" Then sSQL = sSQL & sTemp
            nTemp = nTemp + 1
            sTemp = sGetINIString(sINIPath, sSection, "SQL" & nTemp, "")
        Wend
        frmWait.ProgressBar1.Max = 2000 'Set arbitrary value on progress bar
    End If
    
    
    'Actually open the recordset
    If sSQL = "" Then
        Set rstemp = dbSQL.OpenRecordset("Select * from " & sSQLTable, dbOpenSnapshot, dbForwardOnly)
    Else
        Set rstemp = dbSQL.OpenRecordset(sSQL, dbOpenSnapshot, dbForwardOnly + dbSQLPassThrough)
    End If
    
    'Open Local table
    Set aTable = dbLocal.OpenRecordset(sLocalTable, dbOpenTable)
    nFields = rstemp.Fields.Count - 1

    'Transfer each record
    While Not rstemp.EOF
        aTable.AddNew
        For nTemp = 0 To nFields
            aTable.Fields(nTemp) = rstemp.Fields(nTemp)
        Next nTemp
        rstemp.MoveNext
        nCount = nCount + 1
        frmWait.ProgressBar1.Value = nCount
        aTable.Update
    Wend
    rstemp.Close
    aTable.Close
    Exit Sub

TRTError:
    WriteErrMsg "TRT-Error " & Err & ": " & Error
    End
    Exit Sub

End Sub
Sub CreateTable(ByRef tbl As TableDef, sTableName As String, sINISection As String)
'This function creates an empty table in an Access database
    
    Dim nFields As Integer      'Number of Fields
    Dim Fd() As New Field       'Array of fields
    Dim nCurField As Integer    'Counter for current field
    Dim sTemp As String
       
 On Error GoTo CRTError:
 
    nFields = CInt(sGetINIString(sINIPath, sINISection, "Fields", "0"))
    If nFields = 0 Then Exit Sub
    ReDim Fd(1 To nFields)
   
    tbl.Name = sTableName
    
    For nCurField = 1 To nFields
        Fd(nCurField).Name = sGetINIString(sINIPath, sINISection, "Fd" & nCurField & "Name", "ERROR" & nCurField)
        sTemp = sGetINIString(sINIPath, sINISection, "Fd" & nCurField & "Type", "TEXT")
        Select Case sTemp
            Case "DOUBLE"
            Fd(nCurField).Type = dbDouble
            Case "MEMO"
            Fd(nCurField).Type = dbMemo
            'VB4/5 only
            Fd(nCurField).AllowZeroLength = True
            Case "BYTE"
            Fd(nCurField).Type = dbByte
            Case "INTEGER"
            Fd(nCurField).Type = dbInteger
            Case "DATE"
            Fd(nCurField).Type = dbDate
            Fd(nCurField).Required = False
            
            Case Else 'Text
            Fd(nCurField).Type = dbText
            Fd(nCurField).Size = CInt(sGetINIString(sINIPath, sINISection, "Fd" & nCurField & "Size", "50"))
            Fd(nCurField).AllowZeroLength = True
        End Select
    
       tbl.Fields.Append Fd(nCurField)
    Next nCurField
    Exit Sub
CRTError:
    WriteErrMsg "CRT-Error " & Err & ": " & Error
    End
    Exit Sub
End Sub
Sub CreateLocalFile()
'This procedure creates the MDB file itself
    Dim MainTable() As New TableDef
    Dim sTemp As String
    Dim nTables As Integer
    Dim nCurTable As Integer
    Dim sSQLTable As String
    Dim sMDBTable As String
    Dim sSection As String
    
On Error GoTo CRLError

    If bFileExists(sDBLocalPath) Then Kill sDBLocalPath
    Set dbLocal = CreateDatabase(sDBLocalPath, dbLangGeneral)

    nTables = CInt(sGetINIString(sINIPath, "Tables", "Tables", "0"))
    ReDim MainTable(1 To nTables)
    
    For nCurTable = 1 To nTables
        sSQLTable = sGetINIString(sINIPath, "tables", "table" & nCurTable & "SQL", "none")
        sMDBTable = sGetINIString(sINIPath, "tables", "table" & nCurTable & "MDB", "none")
        sSection = sGetINIString(sINIPath, "tables", "table" & nCurTable & "INI", "none")
        CreateTable MainTable(nCurTable), sMDBTable, sSection
        dbLocal.TableDefs.Append MainTable(nCurTable)
    Next nCurTable
    Exit Sub

CRLError:
    WriteErrMsg "CRL-Error " & Err & ": " & Error
    End
    Exit Sub
End Sub
Private Sub WriteErrMsg(sMessage As String)
    'Should an error occur, this function writes it to
    'another INI file. I did it this way because the
    'program runs as a scheduled process on a remote
    'machine, so no one would be there to answer an error
    'dialog. There is a second VB program that continuously
    'checks the Error.ini and pages me with the error message.
        
    Dim sErrorINI As String
    sErrorINI = sGetINIString(sINIPath, "General", "ErrorINI", "error.ini")
    writeINIString sErrorINI, "Error", "Message", sMessage
    writeINIString sErrorINI, "Error", "Error", "True"
End Sub
Sub writeINIString(INI$, App$, key$, newstr$)
    Dim i%
    Dim sTmp As String
    
    sTmp = ""
    For i% = 1 To Len(newstr$)
    If Mid$(newstr$, i%, 1) <> Chr$(13) And Mid$(newstr$, i%, 1) <> Chr$(10) Then
        sTmp = sTmp & Mid$(newstr$, i%, 1)
    Else
        sTmp = sTmp & " "
    End If
    
    Next i%
    i% = WritePrivateProfileString(App$, key$, sTmp$, INI$)
End Sub
Function sGetINIString(INI$, App$, key$, Dflt$) As String
    
    Dim sTmp As String * 256
    Dim n As Integer
    Dim i As Integer
    
    n = 255
    sTmp = Space$(n + 1)
    i = GetPrivateProfileString(App$, key$, Dflt$, sTmp, n, INI$)
    sGetINIString = Left$(sTmp, i)

End Function
Function bFileExists(sPath As String) As Boolean
    Dim x As Integer
    x = FreeFile

    On Error Resume Next
    Open sPath For Input As x
    
    If Err = 0 Then
        bFileExists = True
    Else
        bFileExists = False
    End If
    
    Close x

End Function

